"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Settings_1 = __importDefault(require("../utils/Settings"));
const exported_helpers_1 = require("../utils/exported_helpers");
const helpers_1 = require("../utils/helpers");
class FavoriteArchive {
    constructor() {
        this._index = {};
    }
    add(favs) {
        for (const f of favs) {
            if ('like' in f) {
                this._index[f.like.tweetId] = f.like;
            }
            else {
                this._index[f.tweetId] = f;
            }
        }
        this._all = undefined;
        this._date_index = undefined;
        this.fav_set = undefined;
    }
    /**
     * Check if {tweet_id} has been favorited.
     */
    has(tweet_id) {
        return tweet_id in this._index;
    }
    /**
     * Get favorites informations of {tweet_id}.
     *
     * If {tweet_id} is not favorited, returns `undefined`.
     */
    single(tweet_id) {
        return this._index[tweet_id];
    }
    /**
     * @deprecated Use `.single()` instead.
     */
    get(tweet_id) {
        return this.single(tweet_id);
    }
    month(month, year) {
        const m = Number(month);
        const y = Number(year);
        const index = this.index;
        if (year in index) {
            if (month in index[year]) {
                return Object.values(index[year][month]);
            }
        }
        return [];
    }
    fromThatDay(start) {
        start = start instanceof Date ? start : new Date;
        const now_m = start.getMonth();
        const now_d = start.getDate();
        const favorites = [];
        if (this._date_index || Settings_1.default.ENABLE_CACHE) {
            const index = this._date_index;
            for (const year in index) {
                for (const month in index[year]) {
                    if (Number(month) === now_m + 1) {
                        // Month of interest
                        helpers_1.safePusher(favorites, Object
                            .values(index[year][month])
                            .filter(t => exported_helpers_1.dateFromFavorite(t).getDate() === now_d));
                    }
                }
            }
        }
        else {
            const now_y = start.getFullYear();
            for (const id in this._index) {
                const tweet = this._index[id];
                const date = exported_helpers_1.dateFromFavorite(tweet);
                if (date.getDate() !== now_d) {
                    continue;
                }
                if (date.getMonth() !== now_m) {
                    continue;
                }
                if (date.getFullYear() !== now_y) {
                    continue;
                }
                favorites.push(tweet);
            }
        }
        return favorites;
    }
    *[Symbol.iterator]() {
        yield* this.all;
    }
    /**
     * Get all the registred favorites
     */
    get all() {
        if (this._all)
            return this._all;
        if (Settings_1.default.ENABLE_CACHE) {
            return this._all = exported_helpers_1.sortFavorites(Object.values(this._index), "asc");
        }
        return exported_helpers_1.sortFavorites(Object.values(this._index), "asc");
    }
    /**
     * Favorites sorted by year then months.
     *
     * Favs on `2010-10` are favorites than cannot be date classed.
     */
    get index() {
        if (this._date_index)
            return this._date_index;
        const index = {};
        for (const fav of this.all) {
            const d = exported_helpers_1.dateFromFavorite(fav);
            const year = d.getFullYear(), month = d.getMonth() + 1;
            if (!(year in index)) {
                index[year] = {};
            }
            if (!(month in index[year])) {
                index[year][month] = {};
            }
            index[year][month][fav.tweetId] = fav;
        }
        if (Settings_1.default.ENABLE_CACHE) {
            return this._date_index = index;
        }
        return index;
    }
    /**
     * Get all favorited tweets in the archive, inside a Set for quick check (like old `archive.extended_gdpr.favorites`).
     */
    get registred() {
        if (this.fav_set) {
            return this.fav_set;
        }
        return this.fav_set = new Set(Object.keys(this._index));
    }
    /**
     * Number of favorites.
     */
    get length() {
        return this.all.length;
    }
    /** `true` if this archive's favorites supported "extended" properties: `.fullText` and `.expandedUrl`. */
    get has_extended_favorites() {
        if (this.length) {
            return 'fullText' in this.all[0];
        }
        return false;
    }
}
exports.FavoriteArchive = FavoriteArchive;
